/* GemRB - Infinity Engine Emulator
 * Copyright (C) 2003 The GemRB Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: DataFileMgr.h 4773 2007-08-17 17:42:55Z avenger_teambg $
 *
 */

/**
 * @file DataFileMgr.h
 * Declares DataFileMgr class, abstract loader for .INI files
 * @author The GemRB Project
 */


#ifndef DATAFILEMGR_H
#define DATAFILEMGR_H

#include "Plugin.h"
#include "DataStream.h"

#ifdef WIN32

#ifdef GEM_BUILD_DLL
#define GEM_EXPORT __declspec(dllexport)
#else
#define GEM_EXPORT __declspec(dllimport)
#endif

#else
#define GEM_EXPORT
#endif

/**
 * @class DataFileMgr
 * Abstract loader for .INI files
 */

class GEM_EXPORT DataFileMgr : public Plugin {
public:
	DataFileMgr(void);
	virtual ~DataFileMgr(void);
	virtual bool Open(DataStream* stream, bool autoFree = false) = 0;
	virtual int GetTagsCount() = 0;
	virtual const char* GetTagNameByIndex(int index) = 0;
	virtual int GetKeysCount(const char* Tag) = 0;
	virtual const char* GetKeyNameByIndex(const char* Tag, int index) = 0;
	virtual const char* GetKeyAsString(const char* Tag, const char* Key,
		const char* Default) = 0;
	virtual const int GetKeyAsInt(const char* Tag, const char* Key,
		const int Default) = 0;
	virtual const bool GetKeyAsBool(const char* Tag, const char* Key,
		const bool Default) = 0;
};

#endif
